package edu.mermet.tp8;


import java.awt.event.*;
import java.io.*;
import java.nio.file.*;
import java.util.*;
import javax.swing.*;

import edu.mermet.tp8.fenetres.*;
import utilisateur.InfoUtilisateur;

/**
 *
 * @author brunomermet
 */
public class Application extends JFrame {
    private static final String[] NOM_SUGGESTIONS = new String[] {"quitter", "diaporama", "conversion", "texte", "boutons"};
    private static final String[] NOM_BOUTONS = new String[] {"Conversion Celsius/Farenheit", "Saisie de texte", "Diaporama", "Boutons"};

    private static InfoUtilisateur info;

    private JInternalFrame conversion;
    private JInternalFrame texte;
    private JInternalFrame diaporama;
    private JInternalFrame boutons;
    private Action actionAfficherConversion;
    private Action actionAfficherTexte;
    private Action actionAfficherDiaporama;
    private Action actionAfficherBoutons;
    private Action actionAfficherCommentFaire;
    private Action actionAfficherConfiguration;
    private JMenu menuApplication;
    private JMenuItem itemConversion;
    private JMenuItem itemTexte;
    private JMenuItem itemDiaporama;
    private JMenuItem itemBoutons;
    private JDialog frameAide;
    private JDialog frameConfiguration;

    public Application() {
        super("multi-fenêtres");
        this.setContentPane(new JDesktopPane());

        initialiserInfoUtilisateur();

        // ****** Barre de menu ******
        JMenuBar barre = new JMenuBar();

        // ------ menu Fichier ------
        JMenu menuFichier = new JMenu("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F);
        JMenuItem quitter = new JMenuItem("Quitter");
        quitter.addActionListener (new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent aev) {
                info.augmenterNiveau(1.0);
                majApplicationsAffichees();
                System.exit(0);
            }
        });
        quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        menuFichier.add(quitter);
        barre.add(menuFichier);
        this.setJMenuBar(barre);

        // ------ menu Applications ------
        this.menuApplication = new JMenu("Applications");
        menuApplication.setMnemonic(KeyEvent.VK_A);
        actionAfficherConversion = new ActionAfficherConversion();
        this.itemConversion = new JMenuItem(actionAfficherConversion);
        actionAfficherTexte = new ActionAfficherTexte();
        this.itemTexte = new JMenuItem(actionAfficherTexte);
        actionAfficherDiaporama = new ActionAfficherDiaporama();
        this.itemDiaporama = new JMenuItem(actionAfficherDiaporama);
        actionAfficherBoutons = new ActionAfficherBoutons();
        this.itemBoutons = new JMenuItem(actionAfficherBoutons);
        majApplicationsAffichees();
        barre.add(menuApplication);

        // ------ menu Aide ------
        JMenu menuAide = new JMenu("Aide");
        menuAide.setMnemonic(KeyEvent.VK_A);
        actionAfficherCommentFaire = new ActionAfficherCommentFaire();
        JMenuItem itemCommentFaire = new JMenuItem(actionAfficherCommentFaire);
        menuAide.add(itemCommentFaire);

        actionAfficherConfiguration = new ActionAfficherConfiguration();
        JMenuItem itemConfiguration = new JMenuItem(actionAfficherConfiguration);
        menuAide.add(itemConfiguration);
        barre.add(menuAide);

        // ****** Fin barre de menu ******
        
        // ****** Création des fenêtres ******
        // ------ fenêtre conversion ------
        conversion = new FenetreConversion(this,actionAfficherConversion);
        this.add(conversion);
        // ------ fenêtre texte ------
        texte = new FenetreTexte(actionAfficherTexte);
        this.add(texte);
        // ------ fenêtre diaporama ------
        diaporama = new FenetreDiaporama(actionAfficherDiaporama);
        this.add(diaporama);
        // ------ fenêtre boutons ------
        boutons = new FenetreBoutons(this,actionAfficherBoutons);
        this.add(boutons);
        // ------ frame aide ------
        frameAide = new FrameAide(actionAfficherCommentFaire, this);
        // ------ frame configuration ------
        frameConfiguration = new FrameConfiguration(actionAfficherConfiguration, this);

        // ****** Fin création fenêtres ******
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(600,300);
        this.setLocationRelativeTo(null);
        setVisible(true);

        // ------ affichage de la suggestion du jour si nécessaire ------
        if (!toutesSuggestionsExclues())
            new FenetreSuggestion(this);
    }

    public int getNbSuggestion()
    {
        double niveau = Double.parseDouble(info.getDonnee("niveau"));

        int nbSuggestions = 1;

        if (niveau >= 2.0) nbSuggestions += 1;
        if (niveau >= 4.0) nbSuggestions += 2;
        if (niveau >= 8.0) nbSuggestions += 1;

        return nbSuggestions;
    }

    public void initialiserInfoUtilisateur()
    {
        info = InfoUtilisateur.getInstance();

        for (int i=0; i<NOM_BOUTONS.length; i++)
            if (info.getDonnee(NOM_BOUTONS[i]) == null)
                info.sauvegarde(NOM_BOUTONS[i], "Auto");

        if (info.getDonnee("niveau") == null)
            info.sauvegarde("niveau", "1.0");
    }

    public boolean toutesSuggestionsExclues()
    {
        int nbSuggestions = getNbSuggestion();

        for (int i=0; i<nbSuggestions; i++)
        {
            if (info.getDonnee(NOM_SUGGESTIONS[i] + "R") == null)
                return false;

            if (info.getDonnee(NOM_SUGGESTIONS[i] + "D") == null)
                return false;
        }

        return true;
    }

    public void majApplicationsAffichees()
    {
        affichageItem(info.getDonnee(NOM_BOUTONS[0]), itemConversion, 4.0);
        affichageItem(info.getDonnee(NOM_BOUTONS[1]), itemTexte, 4.0);
        affichageItem(info.getDonnee(NOM_BOUTONS[2]), itemDiaporama, 2.0);
        affichageItem(info.getDonnee(NOM_BOUTONS[3]), itemBoutons, 8.0);
    }

    public void affichageItem(String choix, JMenuItem item, double niveauRequis)
    {
        if (choix.equals("Affiché"))
            menuApplication.add(item);
        else if (choix.equals("Caché"))
            menuApplication.remove(item);
        else
        {
            double niveau = Double.parseDouble(info.getDonnee("niveau"));
            
            if (niveau >= niveauRequis)
                menuApplication.add(item);
            else
                menuApplication.remove(item);
        }
    }

    private class ActionAfficherBoutons extends AbstractAction {
        public ActionAfficherBoutons() {
            super("Boutons");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_B);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            info.augmenterNiveau(4.0);
            boutons.setVisible(true);
            enableBoutons(false);
            majApplicationsAffichees();
        }
    }

    private class ActionAfficherDiaporama extends AbstractAction {
        public ActionAfficherDiaporama() {
            super("Diaporama");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_D);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            info.augmenterNiveau(2.0);
            diaporama.setVisible(true);
            enableDiaporama(false);
            majApplicationsAffichees();
        }
    }

    private class ActionAfficherTexte extends AbstractAction {
        public ActionAfficherTexte() {
            super("Saisie de texte");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            info.augmenterNiveau(3.0);
            texte.setVisible(true);
            enableTexte(false);
            majApplicationsAffichees();
        }
    }
    
    public class ActionAfficherConversion extends AbstractAction {
        public ActionAfficherConversion() {
            super("Conversion Celsius/Farenheit");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            info.augmenterNiveau(3.0);
            conversion.setVisible(true);
            enableConversion(false);
            majApplicationsAffichees();
        }
    }

    public class ActionAfficherCommentFaire extends AbstractAction {
        public ActionAfficherCommentFaire() {
            super("Comment faire ?");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_F);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            frameAide.setVisible(true);
            enableCommentFaire(false);
        }
    }

    public class ActionAfficherConfiguration extends AbstractAction {
        public ActionAfficherConfiguration() {
            super("Configuration des menus");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_G);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            frameConfiguration.setVisible(true);
            enableConfiguration(false);
        }
    }

    public void enableConversion(boolean b) {
        actionAfficherConversion.setEnabled(b);
    }

    public void enableTexte(boolean b) {
        actionAfficherTexte.setEnabled(b);
    }

    public void enableDiaporama(boolean b) {
        actionAfficherDiaporama.setEnabled(b);
    }

    public void enableBoutons(boolean b) {
        actionAfficherBoutons.setEnabled(b);
    }

    public void enableCommentFaire(boolean b) {
        actionAfficherCommentFaire.setEnabled(b);
    }

    public void enableConfiguration(boolean b) {
        actionAfficherConfiguration.setEnabled(b);
    }

    public Action getActionAfficherConversion() {
        return actionAfficherConversion;
    }

    public Action getActionAfficherTexte() {
        return actionAfficherTexte;
    }

    public Action getActionAfficherDiaporama() {
        return actionAfficherDiaporama;
    }
    public static void main(String[] args) {
        SwingUtilities.invokeLater(Application::new);
    }

    
}
