package utilisateur;

import java.io.*;
import java.nio.file.*;
import java.util.*;

public class InfoUtilisateur
{
	private static InfoUtilisateur INSTANCE = null;
	private Properties parametres;
	private Path pathXml;
	private static final int LVL_MAX_UTILISATEUR = 32;

	private InfoUtilisateur()
	{
        parametres = new Properties();

		String fileName = System.getProperty("user.home") + "/.ihm",
		       xml = "/" + System.getProperty("user.name") + ".xml";

        Path path = Paths.get(fileName);
        this.pathXml = Paths.get(fileName + xml);

        // ****** Création du fichier dans lequel va être stocké les informations utilisateurs ******
        try
        {
            if (!Files.exists(path))
                Files.createDirectory(path);

            if (!Files.exists(pathXml))
                parametres.storeToXML(new FileOutputStream(pathXml.toString()), "Préférences utilisateur");
        }
        catch(IOException ioe) {System.err.println(ioe);}
	}

	public void sauvegarde(String key, String info)
	{
		try
		{
			parametres.loadFromXML(new FileInputStream(pathXml.toString()));
			parametres.setProperty(key, info);
			parametres.storeToXML(new FileOutputStream(pathXml.toString()), "Préférences utilisateur");
        }
        catch(IOException ioe) {System.err.println(ioe);}
	}

	public String getDonnee(String key)
	{
		try
		{
			parametres.loadFromXML(new FileInputStream(pathXml.toString()));
			return parametres.getProperty(key);
        }
        catch(IOException ioe) {System.err.println(ioe);}

        return null;
	}

	public void augmenterNiveau(double niveauComplexite)
	{
		double niveauUtilisateur = Double.parseDouble(getDonnee("niveau"));
		double augmentationNiveau = 0.0;

		if (niveauUtilisateur < (Math.pow(2.0, niveauComplexite) * 2.0))
			augmentationNiveau = Math.pow(2.0, niveauComplexite) / 10.0;
	
		niveauUtilisateur += augmentationNiveau;
		
		if(niveauUtilisateur > InfoUtilisateur.LVL_MAX_UTILISATEUR)
			niveauUtilisateur = InfoUtilisateur.LVL_MAX_UTILISATEUR;
		
		sauvegarde("niveau", "" + Math.round(niveauUtilisateur));
	}
	
	public static synchronized InfoUtilisateur getInstance()
	{
		if (INSTANCE == null)
		{   INSTANCE = new InfoUtilisateur(); 
		}
		return INSTANCE;
	}
}